@@ -8,7 +8,7 @@
 //! you with methods to get the underlying value if you really need it.
 //!
 //! This crate is meant for storing small serializable data that stores the state of an application
-//! after exit. The underlying data is stored in a `.crf` file on disk. Since all the data is loaded
+//! after exit. Since all the data is loaded
 //! onto memory when calling load on a `Client`, handling large amounts of data is not advised. However,
 //! the data stored on disk has a relatively small footprint and should not take up that much space.
 //!
@@ -16,7 +16,11 @@
 //! for this task, such as `sled`:
 //! <https://github.com/spacejam/sled>
 //!
-//! # Examples
+//! This library is once primarily for my own personal use. I simply decided to open-source it just in case
+//! anyone finds this useful. There are no file headers or data versioning support in the created file,
+//! but the format of the data should remain stable for the forseeable future.
+//!
+//! # Example
 //!
 //! ```ignore
 //! #[derive(Serialize, Deserialize, Debug)]
@@ -38,8 +42,7 @@
 //!     message: "You both suck".to_string(),
 //! };
 //! let messages = vec![msg1, msg2, msg3];
-//! let path = PathBuf::new("messages.crf");
-//! let client: Client<Message> = Client::new(path)?; // If no file is found, a new empty file is created
+//! let client: Client<Message> = Client::new("messages", false)?; // If no file is found, a new empty file is created
 //! client.write(messages)?; // If no file is found, a new file is created and then written to
 //! let messages = client.load()?;
 //! if let Some(data) = messages {
@@ -57,12 +60,10 @@ use serde::{de::DeserializeOwned, Serialize};
 use std::{
     fs::{File, OpenOptions},
     io::{Cursor, ErrorKind, Read, Seek, SeekFrom, Write},
-    path::PathBuf,
+    path::Path,
 };
 use thiserror::Error;
 
-const DATA_VERSION: u32 = 1;
-
 /// This is the main error type of this crate.
 #[derive(Error, Debug)]
 pub enum DatabaseError<T> {
@@ -81,15 +82,6 @@ pub enum DatabaseError<T> {
     /// Serialization/deserialization error for an object.
     #[error(transparent)]
     SerdeError(#[from] bincode::Error),
-    /// Each version of `crio` has its own data version. The data format should remain stable
-    /// in perpetuity unless some unforseen circumstances arises. In that case, the data
-    /// version is incremented by 1. The version is stored in each `.crf` file and if the
-    /// data version in file does not match the data version of the current version of `crio`,
-    /// this error occurs, in which case, you should change the version of `crio` you are using
-    /// for your crate to match the data version of the file you are accessing. Such major
-    /// changes would be documented in the README.
-    #[error("wrong data version: expected {DATA_VERSION}, found {0}")]
-    WrongDataVersion(u32),
 }
 
 #[derive(Error, Debug)]
@@ -119,9 +111,7 @@ const CRC: Crc<u32> = Crc::<u32>::new(&CRC_32_ISO_HDLC);
 
 /// An object that is responsible for handling IO operations with regards to file
 /// opening/closing/writing as well as serialization and deserialization.
-///
-/// **NOTE**: If you would like to append data to the file instead, check out
-/// `AppendClient<T>`.
+/// The main data type of this crate.
 #[derive(Debug)]
 pub struct Client<T: Serialize + DeserializeOwned> {
     file: File,
@@ -133,22 +123,28 @@ where
     T: Serialize + DeserializeOwned,
 {
     /// Creates a new client object. It opens a file if a file with the same name exists or
-    /// creates a new file if it doesn't exist.
-    ///
-    /// **NOTE**: This methods automatically appends a `.crf` file extension to your path if
-    /// the extension is not included in the `PathBuf` passed to this method.
+    /// creates a new file if it doesn't exist. Set the `append` parameter to false if you want to
+    /// overwrite all data while calling `write()` or `write_many()`, or true if you
+    /// simply want to append data to the file.
     ///
     /// # Errors
     ///
     /// - The usual `std::io::Error` if it fails to open or create a new file.
-    pub fn new(mut path: PathBuf) -> Result<Self, DatabaseError<T>> {
-        path.set_extension("crf");
-        let file = OpenOptions::new()
-            .read(true)
-            .create(true)
-            .write(true)
-            .truncate(true)
-            .open(path)?;
+    pub fn new<P: AsRef<Path>>(path: P, append: bool) -> Result<Self, DatabaseError<T>> {
+        let file = if append {
+            OpenOptions::new()
+                .read(true)
+                .create(true)
+                .append(true)
+                .open(path.as_ref())?
+        } else {
+            OpenOptions::new()
+                .read(true)
+                .create(true)
+                .write(true)
+                .truncate(true)
+                .open(path.as_ref())?
+        };
         Ok(Self {
             file,
             _phantom: std::marker::PhantomData::default(),
@@ -164,100 +160,6 @@ where
     /// returned, in which you can get the underlying deserialized objects via the
     /// method `into_inner()`.
     ///
-    /// - `DatabaseError<T>::WrongDataVersion` error occurs if the file you are accessing
-    /// is created by a different version of this crate that uses a different file
-    /// format.
-    ///
-    /// - `bincode::Error` occurs if the deserializer fails to deserialize bytes from
-    /// the file to your requested object. In that case, the most probable reason
-    /// is that the data in that file stores some other data type and you are
-    /// attempting to deserialize it to the wrong data type.
-    ///
-    /// - The usual `std::io::Error` such as `ErrorKind::UnexpectedEof` if the file
-    /// that is being accessed is malformed and there are no more bytes to be read
-    /// when the method is expecting more data.
-    pub fn load(&mut self) -> Result<Option<Vec<T>>, DatabaseError<T>> {
-        let mut buf = Vec::new();
-        self.file.read_to_end(&mut buf)?;
-        if buf.is_empty() {
-            return Ok(None);
-        }
-        let result = binary_to_vec(buf)?;
-        Ok(Some(result))
-    }
-
-    /// Writes the provided serializable objects to disk. If no file is found,
-    /// a new file will be created and written to.
-    ///
-    /// **NOTE**: Take note that if a file with the same name exists, this method
-    /// overwrites the data in the file with the data you provided. If you merely
-    /// want to append data, use `AppendClient<T>` instead.
-    ///
-    /// # Errors
-    ///
-    /// - `DatabaseError<T>::DataTooLarge` occurs when an object you are inserting
-    /// takes up more space than `u32::MAX` bytes. In that case, seek help.
-    ///
-    /// - The usual `std::io::Error` such as `ErrorKind::UnexpectedEof` if the file
-    /// that is being accessed is malformed and there are no more bytes to be read
-    /// when the method is expecting more data.
-    ///
-    /// - Serialization errors when the data provided fails to serialize for some reason.
-    pub fn write(&mut self, data: &[T]) -> Result<(), DatabaseError<T>> {
-        let buf = vec_to_binary(data)?;
-        self.file.write_all(&buf)?;
-        Ok(())
-    }
-}
-
-/// This client does the same thing as `Client<T>` with a few notable differences. The `write()`
-/// method merely appends data to an existing file if it exists instead of overwriting the data.
-/// There are also two write methods: `write_many()` which accepts a vector of objects, and
-/// `write()`, which only accepts a single object.
-pub struct AppendClient<T> {
-    file: File,
-    _phantom: std::marker::PhantomData<T>,
-}
-
-impl<T> AppendClient<T>
-where
-    T: Serialize + DeserializeOwned,
-{
-    /// Creates a new client object. It opens a file if a file with the same name exists or
-    /// creates a new file if it doesn't exist.
-    ///
-    /// **NOTE**: This methods automatically appends a `.crf` file extension to your path if
-    /// the extension is not included in the `PathBuf` passed to this method.
-    ///
-    /// # Errors
-    ///
-    /// - The usual `std::io::Error` if it fails to open or create a new file.
-    pub fn new(mut path: PathBuf) -> Result<Self, DatabaseError<T>> {
-        path.set_extension("crf");
-        let file = OpenOptions::new()
-            .read(true)
-            .create(true)
-            .append(true)
-            .open(path)?;
-        Ok(Self {
-            file,
-            _phantom: std::marker::PhantomData::default(),
-        })
-    }
-
-    /// Returns a vector of the deserialized object. If the file is empty, this method
-    /// returns `Ok(None)`.
-    ///
-    /// # Errors
-    ///
-    /// - If a checksum mismatch occurs, a `DataPoisonError<T>` is
-    /// returned, in which you can get the underlying deserialized objects via the
-    /// method `into_inner()`.
-    ///
-    /// - `DatabaseError<T>::WrongDataVersion` error occurs if the file you are accessing
-    /// is created by a different version of this crate that uses a different file
-    /// format.
-    ///
     /// - `bincode::Error` occurs if the deserializer fails to deserialize bytes from
     /// the file to your requested object. In that case, the most probable reason
     /// is that the data in that file stores some other data type and you are
@@ -277,7 +179,7 @@ where
         Ok(Some(result))
     }
 
-    /// Writes the provided serializable objects to disk without overwriting any of the previous data. If no file is found,
+    /// Writes the provided serializable objects to disk. If no file is found,
     /// a new file will be created and written to.
     ///
     /// # Errors
@@ -296,7 +198,7 @@ where
         Ok(())
     }
 
-    /// Writes the provided serializable object to disk without overwriting any of the previous data. If no file is found,
+    /// Writes the provided serializable object to disk. If no file is found,
     /// a new file will be created and written to.
     ///
     /// # Errors
@@ -317,7 +219,6 @@ where
 fn binary_to_vec<T: DeserializeOwned>(raw_data: Vec<u8>) -> Result<Vec<T>, DatabaseError<T>> {
     let mut is_corrupted = None;
     let mut f = Cursor::new(raw_data);
-    validate_data_scheme(&mut f)?;
     let mut result = Vec::new();
     loop {
         let raw_doc = process_document(&mut f);
@@ -344,14 +245,6 @@ fn binary_to_vec<T: DeserializeOwned>(raw_data: Vec<u8>) -> Result<Vec<T>, Datab
     Ok(result)
 }
 
-fn validate_data_scheme<R: Read, T>(f: &mut R) -> Result<(), DatabaseError<T>> {
-    let saved_version = f.read_u32::<LittleEndian>()?;
-    if saved_version != DATA_VERSION {
-        return Err(DatabaseError::WrongDataVersion(saved_version));
-    }
-    Ok(())
-}
-
 fn process_document<R: Read>(f: &mut R) -> Result<Vec<u8>, InnerDatabaseError> {
     let saved_checksum = f.read_u32::<LittleEndian>()?;
     let data_len = f.read_u32::<LittleEndian>()?;
@@ -367,7 +260,6 @@ fn process_document<R: Read>(f: &mut R) -> Result<Vec<u8>, InnerDatabaseError> {
 
 fn vec_to_binary<T: Serialize>(data: &[T]) -> Result<Vec<u8>, DatabaseError<T>> {
     let mut buf = Cursor::new(Vec::new());
-    buf.write_u32::<LittleEndian>(DATA_VERSION)?;
     for document in data {
         let raw_data = bincode::serialize(&document)?;
         let data_len = raw_data.len();
@@ -382,8 +274,8 @@ fn vec_to_binary<T: Serialize>(data: &[T]) -> Result<Vec<u8>, DatabaseError<T>>
     Ok(buf.into_inner())
 }
 
-/// This errors occurs due to a checksum mismatch. Thus it is important to backup your
-/// `.crf` files periodically to prevent data loss. However, you can still get the underlying
+/// This error occurs due to a checksum mismatch. Therefore it is important to backup your
+/// files periodically to prevent data loss. However, you can still get the underlying
 /// objects if you are sure only one or two objects are malformed via the `into_inner()` method
 /// or its equivalents, in which case count your lucky stars as `serde` is still able to
 /// deserialize your objects, or that the saved checksum is the one that is corrupted instead
